#undef gettimeofday
#undef memcpy
#undef memset
#undef strerror

#include "config.h"

#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif
#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#ifdef HAVE_SYS_TIMEB_H
# include <sys/timeb.h>
#endif
#ifdef HAVE_STDDEF_H
# include <stddef.h>
#else
# ifndef NULL
#  define NULL ((void *)0)
# endif
#endif
#ifdef HAVE_STRING_H
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#ifdef HAVE_MEMORY_H
# include <memory.h>
#endif
#if defined(HAVE_MALLOC_H) && !defined(HAVE_STDLIB_H)
# include <malloc.h>
#else
# include <stdlib.h>
#endif
#ifdef HAVE_CTYPE_H
# include <ctype.h>
#endif

#include "common.h"

#if !defined(HAVE_GETTIMEOFDAY) && defined(HAVE_FTIME)
extern gettimeofday(struct timeval * tv, struct timezone * tz);
{
    if (!tv)
    {
        errno = EFAULT;
        return -1;
    }

    struct timeb tb;

    ftime(&tb); /* FIXME: some versions are void return others int */

    tv->tv_sec  = tb.time;
    tv->tv_usec = ((long)tb.millitm)*1000;
    if (tz)
    {
        tz->tz_minuteswest = 0;
        tz->tz_dsttime     = 0;
    }

    return 0;
}
#endif

#ifndef HAVE_MEMCPY
extern void * memcpy(void * dest, void const * src, unsigned long n)
{
#ifdef HAVE_BCOPY
    bcopy(src,dest,n);
    return dest;
#else
/* very slow, your fault for not having memcpy()*/
    unsigned char * td=dest;
    unsigned char * ts=src;
    unsigned long   i;

    if (!td || !ts)
        return NULL;

    for (i=0; i<n; i++)
        td[i] = ts[i];
    return dest;
#endif
}
#endif

#ifndef HAVE_MEMSET
/* very slow, deal with it */
extern void * memset(void * dest, int c, unsigned long n)
{
    unsigned char * temp=dest;
    unsigned long   i;

    if (!temp)
        return NULL;

    for (i=0; i<n; i++)
        temp[i] = (unsigned char)c;
    return dest;
}
#endif

#if !defined(HAVE_STRCASECMP) && !defined(HAVE_STRICMP)
extern int strcasecmp(char const * str1, char const * str2)
{
    unsigned int i;
    int          a;
    int          b;

    if (!str1 || !str2)
        return -1;

    /* some versions of tolower() break when given already lowercase characters */
    for (i=0; str1[i]!='\0' && str2[i]!='\0'; i++)
    {
        if (isupper((int)str1[i]))
            a = (int)tolower((int)str1[i]);
        else
            a = (int)str1[i];

        if (isupper((int)str2[i]))
            b = (int)tolower((int)str2[i]);
        else
            b = (int)str2[i];

        if (a<b)
            return -1;
        if (a>b)
            return +1;
    }

    return 0;
}
#endif

#ifndef HAVE_STRDUP
extern char * strdup(char const * str)
{
    char * out;

    if (!str)
        return NULL;
    if (!(out = malloc(strlen(str)+1)))
        return NULL;
    strcpy(out,str);
    return out;
}
#endif

#ifndef HAVE_STRERROR
extern char const * strerror(int errornum)
{
    if (errornum==0)
	return "No error";
#ifdef EPERM
    if (errornum==EPERM)
	return "Operation not permitted";
#endif
#ifdef ENOENT
    if (errornum==ENOENT)
	return "No such file or directory";
#endif
#ifdef ESRCH
    if (errornum==ESRCH)
	return "No such process";
#endif
#ifdef EINTR
    if (errornum==EINTR)
	return "Interrupted system call";
#endif
#ifdef EIO
    if (errornum==EIO)
	return "I/O error";
#endif
#ifdef ENXIO
    if (errornum==EIO)
	return "No such device or address";
#endif
#ifdef EBADF
    if (errornum==EBADF)
	return "Bad file number";
#endif
#ifdef EAGAIN
    if (errornum==EAGAIN)
	return "Try again";
#endif
#ifdef ENOMEM
    if (errornum==ENOMEM)
	return "Out of memory";
#endif
#ifdef EACCES
    if (errornum==EACCES)
	return "Permission denied";
#endif
#ifdef EFAULT
    if (errornum==EFAULT)
	return "Bad address";
#endif
#ifdef EBUSY
    if (errornum==EBUSY)
	return "Device or resource busy";
#endif
#ifdef EEXIST
    if (errornum==EEXIST)
	return "File exists";
#endif
#ifdef EXDEV
    if (errornum==EXDEV)
	return "Cross-device link";
#endif
#ifdef EDEADLK
    if (errornum==EXDEV)
	return "Resource deadlock would occur";
#endif
#ifdef EDEADLOCK
    if (errornum==EDEADLOCK)
	return "Resource deadlock would occur";
#endif
#ifdef ENODEV
    if (errornum==ENODEV)
	return "No such device";
#endif
#ifdef ENOTDIR
    if (errornum==ENOTDIR)
	return "Not a directory";
#endif
#ifdef EISDIR
    if (errornum==EISDIR)
	return "Is a directory";
#endif
#ifdef EINVAL
    if (errornum==EINVAL)
	return "Invalid argument";
#endif
#ifdef ENFILE
    if (errornum==ENFILE)
	return "Too many open files in system";
#endif
#ifdef EMFILE
    if (errornum==EMFILE)
	return "Too many open files";
#endif
#ifdef ENOTTY
    if (errornum==ENOTTY)
	return "Not a typewriter";
#endif
#ifdef ETXTBSY
    if (errornum==ETXTBSY)
	return "Text file busy";
#endif
#ifdef EFBIG
    if (errornum==EFBIG)
	return "File too large";
#endif
#ifdef ENOSPC
    if (errornum==ENOSPC)
	return "No space left on device";
#endif
#ifdef ESPIPE
    if (errornum==ESPIPE)
	return "Illegal seek";
#endif
#ifdef EROFS
    if (errornum==EROFS)
	return "Read-only file system";
#endif
#ifdef EMLINK
    if (errornum==EMLINK)
	return "Too many links";
#endif
#ifdef EPIPE
    if (errornum==EPIPE)
	return "Broken pipe";
#endif
#ifdef EDOM
    if (errornum==EDOM)
	return "Math argument out of domain of func";
#endif
#ifdef ERANGE
    if (errornum==ERANGE)
	return "Math result not representable";
#endif
#ifdef ENAMETOOLONG
    if (errornum==ENAMETOOLONG)
	return "File name too long";
#endif
#ifdef ENOLCK
    if (errornum==ENOLCK)
	return "No record locks avaliable";
#endif
#ifdef ENOSYS
    if (errornum==ENOSYS)
	return "Function not implemented";
#endif
#ifdef ENOTEMPTY
    if (errornum==ENOTEMPTY)
	return "Directory not empty";
#endif
#ifdef ELOOP
    if (errornum==ELOOP)
	return "Too many symbolic links encountered";
#endif
#ifdef EHOSTDOWN
    if (errornum==EHOSTDOWN)
	return "Host is down";
#endif
#ifdef EHOSTUNREACH
    if (errornum==EHOSTUNREACH)
	return "No route to host";
#endif
#ifdef EALREADY
    if (errornum==EALREADY)
	return "Operation already in progress";
#endif
#ifdef EINPROGRESS
    if (errornum==EINPROGRESS)
	return "Operation now in progress";
#endif
#ifdef ESTALE
    if (errornum==ESTALE)
	return "Stale NFS filehandle";
#endif
#ifdef EDQUOT
    if (errornum==EDQUOT)
	return "Quota exceeded";
#endif
#ifdef EWOULDBLOCK
    if (errornum==EWOULDBLOCK)
	return "Operation would block";
#endif
#ifdef ECOMM
    if (errornum==ECOMM)
	return "Communication error on send";
#endif
#ifdef EPROTO
    if (errornum==EPROTO)
	return "Protocol error";
#endif
#ifdef EPROTONOSUPPORT
    if (errornum==EPROTONOSUPPORT)
	return "Protocol not supported";
#endif
#ifdef ESOCKTNOSUPPORT
    if (errornum==ESOCKTNOSUPPORT)
	return "Socket type not supported";
#endif
#ifdef ESOCKTNOSUPPORT
    if (errornum==EOPNOTSUPP)
	return "Operation not supported";
#endif
#ifdef EPFNOSUPPORT
    if (errornum==EPFNOSUPPORT)
	return "Protocol family not supported";
#endif
#ifdef EAFNOSUPPORT
    if (errornum==EAFNOSUPPORT)
	return "Address family not supported by protocol family";
#endif
#ifdef EADDRINUSE
    if (errornum==EADDRINUSE)
	return "Address already in use";
#endif
#ifdef EADDRNOTAVAIL
    if (errornum==EADDRNOTAVAIL)
	return "Cannot assign requested address";
#endif
#ifdef ENETDOWN
    if (errornum==ENETDOWN)
	return "Network is down";
#endif
#ifdef ENETUNREACH
    if (errornum==ENETUNREACH)
	return "Network is unreachable";
#endif
#ifdef ENETRESET
    if (errornum==ENETRESET)
	return "Network dropped connection on reset";
#endif
#ifdef ECONNABORTED
    if (errornum==ECONNABORTED)
	return "Software caused connection abort";
#endif
#ifdef ECONNRESET
    if (errornum==ECONNRESET)
	return " Connection reset by peer";
#endif
#ifdef ENOBUFS
    if (errornum==ENOBUFS)
	return "No buffer space available";
#endif
#ifdef EISCONN
    if (errornum==EISCONN)
	return "Socket is already connected";
#endif
#ifdef ENOTCONN
    if (errornum==ENOTCONN)
	return "Socket is not connected";
#endif
#ifdef ESHUTDOWN
    if (errornum==ESHUTDOWN)
	return " Cannot send after socket shutdown";
#endif
#ifdef ETIMEDOUT
    if (errornum==ETIMEDOUT)
	return "Connection timed out";
#endif
#ifdef ECONNREFUSED
    if (errornum==ECONNREFUSED)
	return "Connection refused";
#endif
    return "Unknown error";
}
#endif

#ifndef HAVE_STRNCASECMP
extern int strncasecmp(char const * str1, char const * str2, unsigned int cnt)
{
    unsigned int i;
    int          a;
    int          b;

    if (!str1 || !str2)
        return -1;

    /* some versions of tolower() break when given already lowercase characters */
    for (i=0; i<cnt && str1[i]!='\0' && str2[i]!='\0'; i++)
    {
        if (isupper((int)str1[i]))
            a = (int)tolower((int)str1[i]);
        else
            a = (int)str1[i];

        if (isupper((int)str2[i]))
            b = (int)tolower((int)str2[i]);
        else
            b = (int)str2[i];

        if (a<b)
            return -1;
        if (a>b)
            return +1;
    }

    return 0;
}
#endif

#ifndef HAVE_STRSTR
char *
strstr(const char *string, const char *pattern)
{
        char    *cp;
        int     len;

        len = strlen (pattern);

        for (cp = string;cp = strchr (cp, *pattern);) {
                if (strncmp (cp, pattern, len) == 0)
                        return cp;

                cp++;
        }
        return 0;
}
#endif

#ifndef HAVE_STRCASESTR
const unsigned char *
strcasestr(const unsigned char *haystack, const unsigned char *needle)
{
    unsigned int hay_len = strlen(haystack), needle_len = strlen(needle), pos;
    if (hay_len < needle_len) return NULL;
    for (pos=0; pos<(hay_len+1-needle_len); pos++) {
        if (tolower(haystack[pos]) != tolower(*needle)) continue;
        if (!strncasecmp(haystack+pos, needle, needle_len)) return haystack+pos;
    }
    return NULL;
}
#endif
